// Campan
//  2015 Doug Coulter, GPLV2 license
// This is done with state machines and time monitoring instead of delay() to keep loop() responsive.
// Servos are "detached" when not moving for awhile, to prevent jitter and power waste.

#include <Servo.h> 
#define DETACH_MILLIS 500  // time to wait before turning servos off after a command
#define MOVE_MILLIS 50 // time to allow servo to move
#define PAN_DEFAULT 90 // default center
#define TILT_DEFAULT 68
unsigned long pantime; // last time a write servo command happened
unsigned long tilttime;
byte panstate; // state machine state
byte tiltstate;
Servo panservo;  // create servo object to control a servo 
Servo tiltservo; 
int pan;    // variable to store the servo position 
int tilt; // seems to be the center.... 
 
void setup() 
{ 
  Serial.begin(115200);
  panservo.attach(9);  // attaches the servo on pin 9 to the servo object
  tiltservo.attach(10); // up and down 
  pinMode (2,INPUT_PULLUP);  // brn, right (all low true)
  pinMode (3,INPUT_PULLUP);  // red, left
  pinMode (4,INPUT_PULLUP);  // orange up
  pinMode (5,INPUT_PULLUP);  // yellow, down
  pan = PAN_DEFAULT; // center
  tilt = TILT_DEFAULT; // apparent center
  panservo.write(pan);
  tiltservo.write(tilt);
  panstate = tiltstate = 0;
  pantime = tilttime = millis();
} 
////////////////////////// state machines ////////////////////// 
  //servos in pan/tilt reversed as well as switches
  
// state machine for panning servo control 
void panmachine()
{
 switch (panstate)
 {
  case 0: // look for work, detach if no work for too long (save power, prevent jitter/creep)
  if (!digitalRead(2)) 
   {
    pan--;
    if (pan < 0) {pan = 0;}  // protect from user
    panstate = 1; // do the move
    return; // don't bother with the rest
   }
  if (!digitalRead(3)) 
   {
    pan++;
    if (pan > 180) {pan = 180;} // don't go past limit
    panstate = 1;
    return;    
  }  
  
  if (millis() - pantime > DETACH_MILLIS)
  {
    if (panservo.attached()) {panservo.detach();}
  }
  break; // end case 0
  
  case 1: // send position command
    if (!panservo.attached()) {panservo.attach(9);}
    panservo.write(pan);
    pantime = millis(); // remember when sent
    panstate = 2; // wait for move to happen  
  break; // end case 1 send position
  
  case 2: // wait for move before accepting commands again
  if (millis() - pantime > MOVE_MILLIS) {panstate = 0;} // go back to looking for work
  break; // end wait for move
 
 } // end switch 
} // end routine
 
//////////// same for tilt, almost
// state machine for panning servo control 
void tiltmachine()
{
 switch (tiltstate)
 {
  case 0: // look for work, detach if no work for too long (save power, prevent jitter/creep)
  if (!digitalRead(4)) 
   {
    tilt--;
    if (tilt < 20) {tilt = 20;}  // protect from user
    tiltstate = 1; // do the move
    return; // don't bother with the rest
   }
  if (!digitalRead(5)) 
   {
    tilt++;
    if (tilt > 140) {tilt = 140;} // don't go past limit
    tiltstate = 1;
    return;    
  }  
  
  if (millis() - tilttime > DETACH_MILLIS)
  {
    if (tiltservo.attached()) {tiltservo.detach();}
  }
  break; // end case 0
  
  case 1: // send position command
    if (!tiltservo.attached()) {tiltservo.attach(10);}
    tiltservo.write(tilt);
    tilttime = millis(); // remember when sent
    tiltstate = 2; // wait for move to happen  
  break; // end case 1 send position
  
  case 2: // wait for move before accepting commands again
  if (millis() - tilttime > MOVE_MILLIS) {tiltstate = 0;} // go back to looking for work
  break; // end wait for move
 
 } // end switch 
} // end routine
 
//////////////////////// Main ///////////////////////// 
void loop() 
{ 

    panmachine();
    tiltmachine();
      
} 
